//*************************************************************************************************
//
// JUMA-TX500/136 Transmitter Controller
// F4GCB 07.2016
//
// Microchip MPLAB + C30 compiler
// MCU dsPIC30F6014A
//
// Generates a JT9 message from 13 characters string
//
// Acknowledgement :
// The JT9 message algorithm is derived from Fortran files found in the K1JT WSJTX source code.
// Portions of the JT9 message algorithm is the work of Andy Talbot, G4JNT
// Portions of this code is derived from the work of Jason Milldrum, NT7S
//
//*************************************************************************************************

#include <stdio.h>
#include <string.h>	
#include "gene_wspr.h"

// Definitions ------------------------------------------------------------------------------------
#define JT9_1_PERIOD 	576				// JT9-1 period for each symbol 1/(12000/6912) in ms
#define JT9_2_PERIOD 	1280			// JT9-2 period for each symbol 1/(12000/15360) in ms
#define JT9_5_PERIOD 	3413			// JT9-5 period for each symbol 1/(12000/40960) in ms
#define JT9_10_PERIOD 	6912			// JT9-10 period for each symbol 1/(12000/82944) in ms
#define JT9_30_PERIOD 	21000			// JT9-30 period for each symbol 1/(12000/252000) in ms

// External functions -----------------------------------------------------------------------------
extern void play_jt9_sym(int);			// play one JT9 symbol

// External variables -----------------------------------------------------------------------------
int jt9_shift[9];						// TX JT9 shift for DDS

// Local functions --------------------------------------------------------------------------------
static char chr_norm_jt9(char);
static void code_103_bits(char *, unsigned short *);
static int parity(unsigned long);
static void encode_conv(unsigned short *, int *);
static void interleave(int *, int *);
static void pack_gray_sync(int *);

// Local variables --------------------------------------------------------------------------------
static const int sync_vec[85] = {		// 85 bits synchronization vector
	1,1,0,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,1,1,0,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,0,1
	};	
static int symbol[86];					// final symbols after interleaving
static int symbol_idx;					// currently playing JT9 symbol list index
static int symbol_play;					// symbol playing, keyer reserved, 1=playing
static int start_ts;					// starting timeslot available for beacon
static int timeslots;					// number of timeslots passed during beacon on
static int start;						// next timeslot time available
static int wait;						// time waiting before tx


//=================================================================================================
// Return the period in ms for one symbol in function of the speed
// speed=0:JT9-1, speed=1:JT9-2, speed=0:JT9-5, speed=1:JT9-10, speed=0:JT9-30
//=================================================================================================
int jt9_get_period(int speed)
{
	switch (speed)
	{
		case 1:
			return JT9_2_PERIOD;
		break;

		case 2:
			return JT9_5_PERIOD;
		break;

		case 3:
			return JT9_10_PERIOD;
		break;

		case 4:
			return JT9_30_PERIOD;
		break;
	}

	return JT9_1_PERIOD;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Set the symbol playing, keyer reserved, 1=playing
//=================================================================================================
void jt9_set_playing(int on)
{
	if (on)
		symbol_play = 1;
	else
		symbol_play = 0;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Return transmitted JT9 symbol value for display
//=================================================================================================
char *jt9_get_code(char *code)
{
	int x;

	if (symbol_play)
	{
		x = symbol_idx;
		code[0] = 'S';
		code[1] = ' ';
		code[2] = ((x/10)+'0');
		x-=(x/10)*10;
		code[3] = (x +'0');
		code[4] = ':';
		x = symbol[symbol_idx - 1];
		code[5] = (x +'0');
		code[6] = ' ';
		code[7] = ' ';
		code[8] = ' ';
	}
	else
		sprintf(code, "%.2d:%.2d:%.2d ", (wait / 3600), (wait % 3600) / 60, (wait % 3600) % 60);

	return code;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Normalize characters 0..9 A..Z in order 0..35 space + - . / ? in order 36..41
//=================================================================================================
char chr_norm_jt9(char bc) 
{
	char cc = 36;

  	if (bc >= '0' && bc <= '9') cc = bc - '0';
  	if (bc >= 'A' && bc <= 'Z') cc = bc - 'A' + 10;
  	if (bc >= 'a' && bc <= 'z') cc = bc - 'a' + 10; 
	if (bc == ' ' ) cc = 36;
	if (bc == '+' ) cc = 37;
	if (bc == '-' ) cc = 38;
	if (bc == '.' ) cc = 39; 
  	if (bc == '/' ) cc = 40;
	if (bc == '?' ) cc = 41;

  	return(cc);
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Message combined into 103 bits code
//=================================================================================================
void code_103_bits(char *msg, unsigned short *code)
{
	int i;
	unsigned long n1 = 0, n2 = 0, n3 = 0;

	// 13 characters message
	for (i=strlen(msg); i<13; i++)
		msg[i] = ' ';
	msg[13] = 0x00;

	for (i=0; i<5; i++)
		n1 = n1 * 42 + chr_norm_jt9(msg[i]);
 	
	for (i=5; i<10; i++)
		n2 = n2 * 42 + chr_norm_jt9(msg[i]);

	for (i=10; i<13; i++)
		n3 = n3 * 42 + chr_norm_jt9(msg[i]);

	// merge bit 15 and 16 of n3 into n1 and n2, reset bit 15 of n3 
	n1 = (n1 << 1) + ((n3 >> 15) & 1);
	n2 = (n2 << 1) + ((n3 >> 16) & 1);
	n3 = n3 & 0x7FFF;

	// plain text flag
	n3 += 32768;

	// merge n1 value into array code[]
	code[3] = (n1 & 0x0F) << 4;
	n1 = n1 >> 4;
	code[2] = n1 & 0xFF; 
	n1 = n1 >> 8;
	code[1] = n1 & 0xFF;
	n1 = n1 >> 8;
	code[0] = n1 & 0xFF;

	// merge n2 value into array code[]
	code[6] = n2 & 0xFF;
	n2 = n2 >> 8;
	code [5] = n2 & 0xFF;
	n2 = n2 >> 8;
	code[4] = n2 & 0xFF;
	n2 = n2 >> 8;
	code[3] |= n2 & 0x0F;

	// merge n3 value into array code[]
	code[8] = n3 & 0xFF;
	n3 = n3 >> 8;
	code[7] = n3 & 0xFF;

	// the lowest 6 bytes are set to 0 (31 trailing zeros)
	for (i=9; i<13; i++)
		code[i] = 0;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Calculate the single bit parity (XOR)
//=================================================================================================
int parity(unsigned long value)
{
  	int bit_to_1 = 0;
  	while(value != 0)
  	{
    	bit_to_1++;
    	value &= (value - 1);
  	}

  	return (bit_to_1 & 1);
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Convolutional encoding of message array code[] into a 206 bit stream
//=================================================================================================
void encode_conv(unsigned short *code, int *symbol_conv)
{
  	int byte_cnt = 0;
  	int cnt = 0;
  	int i, byte_code;
  	unsigned long shift_register = 0;

  	byte_code = code[0];
	for (i=0; i<103 ;i++)
	{
		// the 103 bits (including the 31 trailing zeros) are read out MSB first
		if (i % 8 == 0 )
		{
      		byte_code = code[byte_cnt];
      		byte_cnt++;
    	}
    	if (byte_code & 0x80) shift_register = shift_register | 1;

    	symbol_conv[cnt++] = parity(shift_register & 0xF2D05351);
    	symbol_conv[cnt++] = parity(shift_register & 0xE4613C47);

    	byte_code = byte_code << 1;
    	shift_register = shift_register << 1;
  	}
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Interleave reorder the 206 data bits
//=================================================================================================
void interleave(int *symbol_conv, int *symbol_inter)
{
  	int address, address_rev ,bit_pos, bit_test;
  	int cnt = 0;

  	for (address=0; address<=255; address++)
	{
    	bit_test = 1;
    	address_rev = 0;
		// bit address reverse eg address = 1 give address_rev = 128
    	for (bit_pos=0; bit_pos<8 ; bit_pos++)
		{
      		if (address & bit_test) address_rev = address_rev | (0x80 >> bit_pos);
      		bit_test = bit_test << 1;
    	}
		// interleaving process
    	if (address_rev < 206)
		{
      		symbol_inter[address_rev] = symbol_conv[cnt];
      		cnt++;
    	}
  	}
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Pack 3 bits into word
//=================================================================================================
void pack_gray_sync(int *symbol_inter)
{
	int i, cnt = 0;
	int symbol_pack[69];

	for (i=0; i<69; i++)
	{
		// Pack 3 bits into word
		symbol_pack[i] = (symbol_inter[cnt] & 1) << 2;
		cnt++;
		symbol_pack[i] |= (symbol_inter[cnt] & 1) << 1;
		cnt++;
		symbol_pack[i] |= symbol_inter[cnt] & 1;
		cnt++;

		// Gray coding
		symbol_pack[i] = symbol_pack[i] ^ (symbol_pack[i] >> 1);
	}

	// merge with the sync vector
	cnt = 0;
	for (i=0; i<85; i++)
	{
		if (sync_vec[i])
			symbol[i] = 0;
		else
		{	
			symbol[i] = symbol_pack[cnt] + 1;
			cnt++;
		}
	}
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Init JT9 beacon
//=================================================================================================
void jt9_init_beacon(const char *msg_in)
{
	int i;
	char msg[14];
	unsigned short int code[13];	// 103 bits message
	int symbol_conv[206];			// symbols after convolutional encoding
	int symbol_inter[206];			// symbols after interleaving
	
	for (i=0; i<strlen(msg_in); i++)
	{
		if (i == 13) break;
		msg[i] = msg_in[i];
	}
	msg[i] = 0x00;

	code_103_bits(msg, code);
	encode_conv(code, symbol_conv);
	interleave(symbol_conv, symbol_inter);
	pack_gray_sync(symbol_inter);

	symbol_play = 0;
	symbol_idx = 0;
	start_ts = 0;
	timeslots = 0;
	start = 0;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// This function is called continuously from main when JT9 beacon is on
// Init beacon must be done before
// speed=0:JT9=1, speed=1:JT9=2, speed=2:JT9=5, speed=3:JT9=10, speed=4:JT9=30
// frame=0:one play, frame=x: one TX timeslots / x timeslots
// timer=O to 3600 s
// return 1 when JT9 message is finished
//=================================================================================================
int jt9_run_beacon(int speed, int frame, int timer)
{
	int delay, timer_reset = 0;

	// wait for machine free
	if (symbol_play)
		return 0;

	// test end of message
	if (symbol_idx == 85)
	{
		symbol_idx = 0;			// JT9 message finished
		return 1;
	}

	switch (speed)
	{
		// timeslot every minute and 1 second
		case 0:
			if (!((timer / 60) % 1) && ((timer % 60) == 1)) start_ts++;
			else start_ts = 0;
			delay = 60;
		break;

		// timeslot every 2 minutes and 1 second
		case 1:
			if (!((timer / 60) % 2) && ((timer % 60) == 1)) start_ts++;
			else start_ts = 0;
			delay = 120;
		break;

		// timeslot every 5 minutes and 1 second
		case 2:
			if (!((timer / 60) % 5) && ((timer % 60) == 1)) start_ts++;
			else start_ts = 0;
			delay = 300;
		break;
		
		// timeslot every 10 minutes and 1 second
		case 3:
			if (!((timer / 60) % 10) && ((timer % 60) == 1)) start_ts++;
			else start_ts = 0;
			delay = 600;
		break;

		// timeslot every 30 minutes and 1 second
		case 4:
			if (!((timer / 60) % 30) && ((timer % 60) == 1)) start_ts++;
			else start_ts = 0;
			delay = 1800;
		break;
	}

	// set the time waiting before tx
	if (timer)
		timer_reset = 0;
	else
		timer_reset++;
	if (start)
	{
		if (start > 3600 && timer_reset == 1) start -= 3600;
		wait = start - timer;
	}
	else
		start = (((timer / delay) + 1) * delay) + 1;

	// enable JT9 TX for the timeslot according to the frame
	if (start_ts == 1)
	{ 
		if (frame == 0)
			symbol_play = 1;
		else if (!(timeslots % frame))
		{
			// reset timeslots counter to avoid overflow
            if (timeslots == 60)
                timeslots = 0;

			// next timeslot time available
			start += frame * delay;
		
			symbol_play = 1;
			timeslots++;
		}
		else
			timeslots++;
	}

	return 0;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Beacon JT9 generator
// 1ms IRQ function, feed beacon JT9 symbols to the keyer code
//=================================================================================================
void jt9_sym_irq(void)
{
	// run logic only when play symbol
	if(symbol_play)
	{
		if (symbol_idx == 85)				// end of timeslot
			symbol_play = 0;
		else
		{
			play_jt9_sym(symbol[symbol_idx]);
			symbol_idx++;
		}
	}
}
//-------------------------------------------------------------------------------------------------
